<?php

/**
 * @file
 *
 * Admin settings.
 */


/**
 * Returns standard config fields.
 *
 * @return array
 *   Predefined configuration.
 */
function lesser_forms_get_config_fields() {
  $config_fields = array(
    'promote',
    'sticky',
    'preview',
    'author',
    'revision_information',
    'path',
  );
  return $config_fields;
}

/**
 * Configuration form.
 */
function lesser_forms_configuration_form($form, &$form_state) {
  $config_fields = lesser_forms_get_config_fields();
  $settings = variable_get('lesser_forms_config', array());

  $user_roles = user_roles();
  $header = _lesser_forms_get_table_header($user_roles);

  $form['table'] = array(
    '#theme' => 'lesser_forms_configuration_form_table',
    '#header' => $header,
    'rows' => array(
      // Make it a tree for easy traversing of the entered values on submission.
      '#tree' => TRUE,
    ),
  );

  // Loop for all standard lesser forms fields.
  foreach ($config_fields as $config_field) {
    // Call custom function to create rows.
    $arr = _lesser_forms_create_rows($config_field);
    // Add empty action column.
    $arr[] = array(
      'Action' => array(),
    );
    $form['table']['rows'][] = $arr;
  }
  // Loop for all custom fields.
  foreach ($settings as $field => $value) {
    if (!in_array($field, $config_fields)) {
      // Call custom function to create rows.
      $arr = _lesser_forms_create_rows($field);
      // Add Remove Button for custom field.
      $arr[] = array(
        '#type' => 'submit',
        '#value' => t('Remove'),
        '#name' => 'remove-button-' . $field,
        '#value_callback' => $field,
        '#submit' => array('lesser_forms_configuration_form_delete_custom_field_submit'),
      );
      $form['table']['rows'][] = $arr;
    }
  }

  // Custom field form elements.
  $form['group_custom_field'] = array(
    '#tree' => TRUE,
    '#type' => 'fieldset',
    '#title' => t('Add Custom Field'),
    '#collapsible' => FALSE,
  );

  $form['group_custom_field']['add_custom_field'] = array(
    '#type' => 'textfield',
    '#autocomplete_path' => 'admin/config/content/lesser_forms/autocomplete',
    '#attributes' => array(
      'placeholder' => t('Custom Field Machine Name'),
    ),
  );
  $form['group_custom_field']['button_add_custom_field'] = array(
    '#value' => t('Add'),
    '#type' => 'submit',
    '#id' => 'button-add-custom-field',
    '#submit' => array('lesser_forms_configuration_form_add_custom_field_submit'),
  );

  $applies_to = variable_get('lesser_forms_applies_to', array());
  // Lesser Forms is active on these form machine names.
  $form['applies_to'] = array(
    '#title' => t('Enabled on'),
    '#type' => 'textarea',
    '#description' => t('Lesser Forms will only be active on the form machine names you insert here. Only one machine name per line. Use new lines to add multiples.') . '<br>' . t('You can use a wildcard "*"-symbol to apply to every standardized form, but keep in mind this is arbitrary.') . '<br>' . t('These content type forms are known by your site already:') . ' ' . _lesser_forms_get_node_types_form(),
    '#default_value' => $applies_to,
  );

  // Define our submit handler.
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Config'),
    '#submit' => array('lesser_forms_configuration_form_submit'),
  );

  // Custom validation handler.
  $form['#validate'][] = 'lesser_forms_configuration_form_validate';

  return $form;
}

/**
 * Form validation.
 */
function lesser_forms_configuration_form_validate(&$form, &$form_state) {
  // Custom validation for adding custom fields.
  if ($form_state['clicked_button']['#id'] == 'button-add-custom-field') {
    $value = $form_state['values']['group_custom_field']['add_custom_field'];
    if ($value == '') {
      form_set_error('group_custom_field][add_custom_field', t('Input for new custom field can not be empty'));
    }
    if (strpos($value, " ") !== FALSE) {
      form_set_error('group_custom_field][add_custom_field', t('Custom field name can not have whitespace.'));
    }
  }
}

/**
 * Submit for field adding.
 */
function lesser_forms_configuration_form_add_custom_field_submit(&$form, &$form_state) {
  $settings = variable_get('lesser_forms_config', array());
  $value = $form_state['values']['group_custom_field']['add_custom_field'];
  $user_roles = user_roles();
  $arr_values = array();

  foreach ($user_roles as $role) {
    $arr_values[$value][$role] = 0;
  }

  $merged_result = array_merge($settings, $arr_values);
  variable_set('lesser_forms_config', $merged_result);

  // @TODO: call submit.
  header('Location: ' . $_SERVER['PHP_SELF']);

}

/**
 * Submit for field deletion.
 */
function lesser_forms_configuration_form_delete_custom_field_submit(&$form, &$form_state) {
  $value = $form_state['clicked_button']['#value_callback'];
  $settings = variable_get('lesser_forms_config', array());
  unset($settings[$value]);
  variable_set('lesser_forms_config', $settings);

  // @TODO: call submit.
  header('Location: ' . $_SERVER['PHP_SELF']);
}

/**
 * Submit for configuration form.
 */
function lesser_forms_configuration_form_submit(&$form, &$form_state) {
  $arr_values = array();
  $values = $form_state['values']['rows'];

  foreach ($values as $row) {
    foreach ($row as $role => $id) {
      // Do not do for remove button row.
      if (is_array($id)) {
        $first_key = key($id);
        $arr_values[$first_key][$role] = $id[$first_key];
      }
    }
  }

  $applies_to = $form_state['values']['applies_to'];
  // Overwrite the variable settings each save.
  variable_set('lesser_forms_config', $arr_values);
  variable_set('lesser_forms_applies_to', $applies_to);


  drupal_set_message(t('System preferences saved!'));
}


/**
 * Prepares the config table header.
 *
 * @param array $user_roles
 *   User roles.
 *
 * @return array
 *   Header for our table.
 */
function _lesser_forms_get_table_header($user_roles) {
  $header = array(t('field'));

  foreach ($user_roles as $id => $entity) {
    $header[] = $entity;
  }
  $header[] = t('Actions');
  return $header;
}

/**
 * Custom function to create rows.
 *
 * @param string $field
 *   Input field.
 *
 * @return array
 *   Return a row.
 */
function _lesser_forms_create_rows($field) {
  $settings = variable_get('lesser_forms_config', array());
  $user_roles = user_roles();

  // First column is always fieldname.
  $arr = array(
    'c1' => array(
      '#markup' => $field,
    ),
  );

  // Loop all user roles and add checkboxes.
  foreach ($user_roles as $role) {
    $arr[$role] = array(
      $field => array(
        '#type' => 'checkbox',
        '#id' => 'table-field-' . $field,
        '#default_value' => isset($settings[$field][$role]) ? $settings[$field][$role] : 0,
      ),
    );
  }

  return $arr;
}

/**
 * Returns all Node Types.
 *
 * @return array
 *   A list of items.
 */
function _lesser_forms_get_node_types_form() {
  $array_keys = array_keys(node_type_get_types());

  // Format the keys.
  $items = array();
  foreach ($array_keys as $item) {
    $items[$item] = $item . '_node_form';
  }

  return implode(', ', $items);
}
